﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using Common.Ajax.Extension;

namespace Common.Ajax.Service
{
    /*使用DataMember统一管理属性和字段*/

    internal abstract class DataMember
    {
        public abstract string Name { get; }
        public abstract Type MemberType { get; }
        public abstract void SetValue(object instance,object value);
        public abstract object GetValue(object instance);
    }

    internal class PropertyMember : DataMember
    {
        private PropertyInfo property;
        public PropertyMember(PropertyInfo property)
        {
            if (property == null)
            {
                throw new ArgumentNullException("property");
            }
            this.property = property;
        }

        public override void SetValue(object instance, object value)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }
            this.property.SetValue(instance, value, null);
        }

        public override object GetValue(object instance)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }
            return this.property.GetValue(instance,null);
        }

        public override string  Name
        {
	        get { return this.property.Name; }
        }

        public override Type MemberType
        {
            get { return this.property.PropertyType; }
        }
    }

    internal class FieldMember : DataMember
    {
        private FieldInfo field;
        public FieldMember(FieldInfo field)
        {
            if (field == null)
            {
                throw new ArgumentNullException("field");
            }
            this.field = field;
        }

        public override void SetValue(object instance, object value)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }
            this.field.SetValue(instance, value);
        }

        public override object GetValue(object instance)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }
            return this.field.GetValue(instance);
        }

        public override string  Name
        {
	        get { return this.field.Name;}
        }

        public override Type MemberType
        {
            get { return this.field.FieldType; }
        }
    }

    internal static class DataMemberManager
    {
        /// <summary>
        /// 获取实例字段/属性集合
        /// </summary>
        /// <param name="type">类型</param>
        /// <returns></returns>
        public static List<DataMember> GetDataMember(Type type)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }
            IEnumerable<PropertyMember> propertyMembers = from property in type.GetProperties(BindingFlags.Instance | BindingFlags.Public)
                                                  select new PropertyMember(property);
            IEnumerable<FieldMember> fieldMembers = from field in type.GetFields(BindingFlags.Instance | BindingFlags.Public)
                                             select new FieldMember(field);
            List<DataMember> members = new List<DataMember>();
            foreach(var property in propertyMembers)
            {
                members.Add(property);
            }
            foreach (var field in fieldMembers)
            {
                members.Add(field);
            }
            return members;
        }
    }
}
